==> f.lux-0.8.25.tgz: package/src/Shader.js <==
import invariant from "invariant";

import AutoShader from "./AutoShader";
import noParentStateErrorMsg from "./noParentStateErrorMsg";
import PropertyFactoryShader from "./PropertyFactoryShader";
import shadowProperty from "./shadowProperty";

import { isSomething } from "akutils";


const readonlyAutoShader = new AutoShader(true);
const readWriteAutoShader = new AutoShader(false);

/*
	Mappings for property names that always have the same type, such as create_on dates.
*/
const defaultShaders = {};

// Instnace variable names
const _automount = Symbol('_autoMount');
const _automountAll = Symbol('automountAll');
const _autoshadow = Symbol('autoShadow');
const _elementShader = Symbol('elementShader');
const _elementShaderAutomount = Symbol('elementShaderAutomount');
const _property = Symbol('property');
const _shaders = Symbol('shaders');


/*
	Combine with PropertyShader now that everything is a property.
*/
export default class Shader {
	constructor(property, autoShadow=false, automountAll=false) {
		this[_property] = property;
		this[_autoshadow] = autoShadow;
		this[_automountAll] = automountAll;

		// initialize instance variables
		this[_automount] = [];
		this[_elementShader] = null;
		this[_shaders] = { };
	}

	static addDefault(name, shader) {
		defaultShaders[name] = shader;
	}

	get automountAll() {
		return this[_automountAll];
	}

	get autoShadow() {
		return this[_autoshadow];
	}

	get elementShader() {
		return this[_elementShader];
	}

	get property() {
		return this[_property];
	}

	add(name, shader, automount) {
		this[_shaders][name] = shader;

		if (automount) {
			this[_automount].push(name);
		}

		return this;
	}

	addPropertyClass(name, PropertyClass, defaults, autoShadow, readonly) {
		const property = this[_property];
		const shaderReadonly = readonly || property.readonly;
		const shader = new PropertyFactoryShader(PropertyClass, property, defaults, autoShadow, shaderReadonly);

		this.add(name, shader);
	}

	addStateType(name, stateType) {
		const property = this[_property];
		const shader = stateType.factory(property);

		this.add(name, shader);
	}

	autoShader() {
		if (this[_autoshadow]) {
			return this[_property].readonly ?readonlyAutoShader :readWriteAutoShader;
		} else {
			return null;
		}
	}

	get(name) {
		return this[_shaders][name];
	}

	isAutomount(name) {
		if (this[_automountAll]) {
			return true;
		} else if (this[_shaders][name]) {
			return this[_automount][name] || this[_shaders][name].shouldAutomount();
		} else if (this[_elementShader]) {
			return this[_elementShaderAutomount] || this[_elementShader].shouldAutomount();
		} else {
			return false;
		}
	}

	remove(name) {
		delete this[_shaders][name];

		// remove the automount setting
		const automountIdx = this[_automount].indexOf(name);

		if (automountIdx != -1) {
			this[_automount].splice(automountId, 1);
		}

		return this;
	}

	setAutomountAll(automount) {
		this[_automountAll] = automount;
	}

	setAutoshadow(value) {
		this[_autoshadow] = !!value;

		return this;
	}

	setElementClass(PropertyClass, defaults, autoShadow, readonly) {
		const property = this[_property];
		const shaderReadonly = readonly || property.readonly;
		const shader = new PropertyFactoryShader(PropertyClass, property, defaults, autoShadow, shaderReadonly);

		return this.setElementShader(shader);
	}

	setElementStateType(stateType) {
		const property = this[_property];
		const shader = stateType.factory(property);

		return this.setElementShader(shader);
	}

	setElementShader(shader, automount) {
		if (this[_elementShader]) {
			console.warn(`Property already has a child shader set - ignoring setElementShader() call - shader=%O`, this);
			return;
		}

		this[_elementShader] = shader;
		this[_elementShaderAutomount] = automount;

		return this;
	}

	/*
		Gets the shader for a child property. This method returns the first shader generated by checking:
			1) Property subclass optional shaderFor() method
			2) shaders registered using add() or addPropertyClass() - named shaders
			3) child shader (registered using setElementShader() )
			4) default shaders (regustered using the static addDefault() method)
			5) auto shader (applicable if autoShadow set to true)

		Parameters:
			name - the name of the child property
			sate - the property's current state (not the child state)
	*/
	shaderFor(name, state) {
		if (this[_property].shaderFor) {
			let shader = this[_property].shaderFor(name, state);

			if (shader) {
				return shader
			}
		}

		// priority order: named, child, default mapping, auto shading
		return this[_shaders][name] ||
			this[_elementShader] ||
			defaultShaders[name] ||
			this.autoShader();
	}

	/*
		Creates the virtual state (shadow) for a state property.
	*/
	shadowProperty(time, name, parentState, parentImpl) {
		const property = this[_property];
		const isRoot = property.isRoot();

		invariant(parentState || isRoot, noParentStateErrorMsg(name, parentImpl));

		// get the initial state for the property
		const currState = isRoot ?parentState :parentState[name];
		const state = property.getInitialState(currState);

		if (!isRoot) {
			parentState[name] = state;
		}

		// add mixins to the property (but only once)
		if (!property.__hasMixins()) {
			let proto = Object.getPrototypeOf(property);
			let mixins = proto.constructor.mixins;
			let mixin;

			if (mixins) {
				for (let i=0, len=mixins.length; i<len; i++) {
					mixin = mixins[i](property);

					property.__addMixin(mixin);
				}
			}
		}

		// create the backing implementation instance
		const ImplClass = property.implementationClass();
		const impl = shadowProperty(time, ImplClass, property, name, state, parentImpl, this);

		// set the current implementation
		property.setImpl(impl);

		return impl;
	}

	/*
		Automounting is an experimental, partially implemented feature.
	*/
	shouldAutomount() {
		const proto = Object.getPrototypeOf(this[_property]);

		return proto.constructor.shouldAutomount && proto.constructor.shouldAutomount();
	}
}

==> f.lux-0.8.46.tgz: package/src/Shader.js <==
import invariant from "invariant";

import AutoShader from "./AutoShader";
import noParentStateErrorMsg from "./noParentStateErrorMsg";
import PropertyFactoryShader from "./PropertyFactoryShader";
import shadowProperty from "./shadowProperty";

import { isSomething } from "akutils";


const readonlyAutoShader = new AutoShader(true);
const readWriteAutoShader = new AutoShader(false);

/*
	Mappings for property names that always have the same type, such as create_on dates.
*/
const defaultShaders = {};

// Instnace variable names
const _automount = Symbol('_autoMount');
const _automountAll = Symbol('automountAll');
const _autoshadow = Symbol('autoShadow');
const _elementShader = Symbol('elementShader');
const _elementShaderAutomount = Symbol('elementShaderAutomount');
const _property = Symbol('property');
const _shaders = Symbol('shaders');


/*
	Combine with PropertyShader now that everything is a property.
*/
export default class Shader {
	constructor(property, autoShadow=false, automountAll=false) {
		this[_property] = property;
		this[_autoshadow] = autoShadow;
		this[_automountAll] = automountAll;

		// initialize instance variables
		this[_automount] = [];
		this[_elementShader] = null;
		this[_shaders] = { };
	}

	static addDefault(name, shader) {
		defaultShaders[name] = shader;
	}

	get automountAll() {
		return this[_automountAll];
	}

	get autoShadow() {
		return this[_autoshadow];
	}

	get elementShader() {
		return this[_elementShader];
	}

	get property() {
		return this[_property];
	}

	add(name, shader, automount) {
		this[_shaders][name] = shader;

		if (automount) {
			this[_automount].push(name);
		}

		return this;
	}

	addPropertyClass(name, PropertyClass, defaults, autoShadow, readonly) {
		const property = this[_property];
		const shaderReadonly = readonly || property.readonly;
		const shader = new PropertyFactoryShader(PropertyClass, property, defaults, autoShadow, shaderReadonly);

		this.add(name, shader);
	}

	addStateType(name, stateType) {
		const property = this[_property];
		const shader = stateType.factory(property);

		this.add(name, shader);
	}

	autoShader() {
		if (this[_autoshadow]) {
			return this[_property].readonly ?readonlyAutoShader :readWriteAutoShader;
		} else {
			return null;
		}
	}

	get(name) {
		return this[_shaders][name];
	}

	has(name) {
		return !!this[_shaders][name];
	}

	isAutomount(name) {
		if (this[_automountAll]) {
			return true;
		} else if (this[_shaders][name]) {
			return this[_automount][name] || this[_shaders][name].shouldAutomount();
		} else if (this[_elementShader]) {
			return this[_elementShaderAutomount] || this[_elementShader].shouldAutomount();
		} else {
			return false;
		}
	}

	remove(name) {
		delete this[_shaders][name];

		// remove the automount setting
		const automountIdx = this[_automount].indexOf(name);

		if (automountIdx != -1) {
			this[_automount].splice(automountIdx, 1);
		}

		return this;
	}

	setAutomountAll(automount) {
		this[_automountAll] = automount;
	}

	setAutoshadow(value) {
		this[_autoshadow] = !!value;

		return this;
	}

	setElementClass(PropertyClass, defaults, autoShadow, readonly) {
		const property = this[_property];
		const shaderReadonly = readonly || property.readonly;
		const shader = new PropertyFactoryShader(PropertyClass, property, defaults, autoShadow, shaderReadonly);

		return this.setElementShader(shader);
	}

	setElementStateType(stateType) {
		const property = this[_property];
		const shader = stateType.factory(property);

		return this.setElementShader(shader);
	}

	setElementShader(shader, automount) {
		if (this[_elementShader]) {
			console.warn(`Property already has a child shader set - ignoring setElementShader() call - shader=%O`, this);
			return;
		}

		this[_elementShader] = shader;
		this[_elementShaderAutomount] = automount;

		return this;
	}

	/*
		Gets the shader for a child property. This method returns the first shader generated by checking:
			1) Property subclass optional shaderFor() method
			2) shaders registered using add() or addPropertyClass() - named shaders
			3) child shader (registered using setElementShader() )
			4) default shaders (regustered using the static addDefault() method)
			5) auto shader (applicable if autoShadow set to true)

		Parameters:
			name - the name of the child property
			sate - the property's current state (not the child state)
	*/
	shaderFor(name, state) {
		if (this[_property].shaderFor) {
			let shader = this[_property].shaderFor(name, state);

			if (shader) {
				return shader
			}
		}

		// priority order: named, child, default mapping, auto shading
		return this[_shaders][name] ||
			this[_elementShader] ||
			defaultShaders[name] ||
			this.autoShader();
	}

	/*
		Creates the virtual state (shadow) for a state property.
	*/
	shadowProperty(time, name, parentState, parentImpl) {
		const property = this[_property];
		const isRoot = property.isRoot();

		invariant(parentState || isRoot, noParentStateErrorMsg(name, parentImpl));

		// get the initial state for the property
		const currState = isRoot ?parentState :parentState[name];
		const state = property.getInitialState(currState);

		if (!isRoot) {
			parentState[name] = state;
		}

		// add mixins to the property (but only once)
		if (!property.__hasMixins()) {
			let proto = Object.getPrototypeOf(property);
			let mixins = proto.constructor.mixins;
			let mixin;

			if (mixins) {
				for (let i=0, len=mixins.length; i<len; i++) {
					mixin = mixins[i](property);

					property.__addMixin(mixin);
				}
			}
		}

		// create the backing implementation instance
		const ImplClass = property.implementationClass();
		const impl = shadowProperty(time, ImplClass, property, name, state, parentImpl, this);

		// set the current implementation
		property.setImpl(impl);

		return impl;
	}

	/*
		Automounting is an experimental, partially implemented feature.
	*/
	shouldAutomount() {
		const proto = Object.getPrototypeOf(this[_property]);

		return proto.constructor.shouldAutomount && proto.constructor.shouldAutomount();
	}
}
